import requests
import json
import time
from datetime import datetime
from urllib.parse import urlencode

class Vault:
    def __init__(self, address = 'https://127.0.0.1/PasswordVault'):
        self.address = address
        self.username = ''
        self.password = ''
        self.timeout = None
        self.verify = True
        self.loginresponse = None
        self.token = ''
        self.authresponse = ''
        self.lastexception = None
        self.sleepafter = -1
        self.dorelogin = False
        self.doreloginwindow = 120
        self.lastlogon = datetime(1970,1,1,0,0,0)
    def hello(self):
        print("Hello, World!", flush = True)
    def setVerify(self, verify):
        self.verify = verify
    def setTimeout(self, timeout):
        self.timeout = timeout
    def setSleepAfter(self, sleepafter):
        self.sleepafter = sleepafter
    def setDoRelogin(self, dorelogin):
        self.dorelogin = dorelogin
    def setDoReloginWindow(self, doreloginwindow):
        self.doreloginwindow = doreloginwindow
    def getLoginResponse(self):
        return self.loginresponse
    def getAuthResponse(self):
        return self.authresponse
    def getLastException(self):
        return self.lastexception
    def getToken(self):
        return self.token
    def getAIMCredential(self, address, appid, safe, object):
        try:
            str_url_aim = 'https://{0}/AIMWebService/api/Accounts?AppID={1}&Safe={2}&Object={3}'.format(address, appid, safe, object)
            response = requests.get(str_url_aim, verify=self.verify, timeout=self.timeout)
            #print('https://{0}/AIMWebService/api/Accounts?AppID={1}&Safe={2}&Object={3}'.format(address, appid, safe, object), flush = True)
            #print(response.text, flush = True)
            #print(response.status_code, flush = True)
            if response.status_code >= 400:
                str_exception = 'ERROR STATUS CODE {0}.\nURL = {1}\nResponse =\n{2}'.format(response.status_code, str_url_aim, response.text)
                raise Exception(str_exception)
            return json.loads(response.text)
        except Exception as e:
            self.lastexception = e
            return None
    def login_cyberark(self, username = None, password = None):
        headers = {
            'Content-Type': 'application/json'
        }
        if username is None or password is None:
            pass
        else:
            self.username = username
            self.password = password
        odata = {
            'username': self.username,
            'password': self.password
        }
        sdata = json.dumps(odata)
        try:
            response = requests.post('{0}/API/auth/Cyberark/Logon'.format(self.address), headers=headers, data=sdata, verify=self.verify, timeout=self.timeout)
            if response.status_code >= 400:
                raise Exception(response.text)
            self.authresponse = response.text
            self.loginresponse = json.loads(response.text)
            self.token = self.loginresponse
            self.lastlogon = datetime.now()
            return True
        except Exception as e:
            self.loginresponse = None
            self.token = ''
            self.lastexception = e
            return False
    def logoff(self):
        if self.token is None or self.token == '':
            return None
        headers = {
            'Authorization': self.token
        }
        try:
            response = requests.post('{0}/API/Auth/Logoff'.format(self.address), headers=headers, verify=self.verify, timeout=self.timeout)
            self.token = ''
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return {}
                #return json.loads(response.text)
        except Exception as e:
            #print(str(e), flush = True)
            self.lastexception = e
            self.token = ''
            return None
    def getAccounts(self, limit = 100):
        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None
        headers = {
            'Authorization': self.token
        }
        try:
            response = requests.get('{0}/api/Accounts?limit={1}'.format(self.address, limit), headers=headers, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return json.loads(response.text)
        except Exception as e:
            self.lastexception = e
            return None

    def getAccountsForSafe(self, safeName = ''):
        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None
        headers = {
            'Authorization': self.token
        }
        try:
            finalresponse = {
                'value': []
            }
            keepworking = True
            offset = 0
            limit = 1000
            while keepworking:
                response = requests.get('{0}/api/Accounts?filter=safeName%20eq%20{1}&limit={2}&offset={3}'.format(self.address, safeName, limit, offset), headers=headers, verify=self.verify, timeout=self.timeout)
                if self.sleepafter > 0:
                    time.sleep(self.sleepafter)
                if response.status_code >= 400:
                    #raise Exception(response.text)
                    keepworking = False
                else:
                    #with open('test_{0}.json'.format(safeName), mode='w', encoding='utf-8') as testfile:
                    #    o = json.loads(response.text)
                    #print(json.dumps(o, indent=4), file=testfile, flush=True)
                    o = json.loads(response.text)
                    if len(o['value']) < 1:
                        keepworking = False
                    else:
                        finalresponse['value'].extend(o['value'])
                        offset = offset + limit
            return finalresponse
        except Exception as e:
            #print(e, flush = True)
            self.lastexception = e
            return None

    def retrievePassword(self, accountId):
        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None
        headers = {
            'Authorization': self.token
        }
        try:
            url = '{0}/api/Accounts/{1}/Password/Retrieve'.format(self.address, accountId)
            data = {
                'reason': 'report'
            }
            response = requests.post(url, headers=headers, data=data, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return json.loads(response.text)
        except Exception as e:
            self.lastexception = e
            return None

    def searchFirstAccountOnSafe(self, search, safeName, limit = 100):
        accounts = self.searchAccountsOnSafe(search, safeName, limit)
        if accounts is None:
            return None
        if accounts['value'] is None or len(accounts['value']) < 1:
            return None
        return accounts['value'][0]


    def searchAccountsOnSafe(self, search, safeName, limit = 100):
        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None
        headers = {
            'Authorization': self.token
        }
        try:
            query = {
                'filter': f'safeName eq {safeName}',
                'search': f'{search}',
                'offset': 0,
                'limit': limit
            }
            url = '{0}/api/Accounts?{1}'.format(self.address, urlencode(query));
            response = requests.get(url, headers=headers, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return json.loads(response.text)
        except Exception as e:
            self.lastexception = e
            return None

    def getAccountsOnSafe(self, safeName, limit = 100):
        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None
        headers = {
            'Authorization': self.token
        }
        try:
            query = {
                'filter': f'safeName eq {safeName}',
                'offset': 0,
                'limit': limit
            }
            url = '{0}/api/Accounts?{1}'.format(self.address, urlencode(query));
            response = requests.get(url, headers=headers, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return json.loads(response.text)
        except Exception as e:
            self.lastexception = e
            return None

    def addAccount(self, safeName, platformId, name, address, username, secret, platformAccountProperties = {}):
        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None

        headers = {
            'Content-Type': 'application/json',
            'Authorization': self.token
        }
        odata = {
            #'name': '{0}@{1}'.format(name, address),
            'address': address,
            #'userName': username,
            'platformId': platformId,
            'safeName': safeName,
            'secretType': 'password',
            'secret': secret,
            'platformAccountProperties': platformAccountProperties
        }

        if name is not None:
            odata['name'] = name

        if username is not None:
            odata['userName'] = username

        sdata = json.dumps(odata)

        try:
            response = requests.post('{0}/api/Accounts'.format(self.address), headers=headers, data=sdata, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return json.loads(response.text)
        # except requests.exceptions.HTTPError as e:
        #     print(e, flush = True)
        #     return None
        except Exception as e:
            self.lastexception = e
            return None

    def updateAccountSingle(self,accountId, op, path, value):

        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None

        headers = {
            'Content-Type': 'application/json',
            'Authorization': self.token
        }
        odata = [
            {
                'op': op,
                'path': path,
                'value': value
            }
        ]
        sdata = json.dumps(odata)

        try:
            response = requests.patch('{0}/api/Accounts/{1}'.format(self.address, accountId), data=sdata, headers=headers, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return {}
        except Exception as e:
            self.lastexception = e
            return None

    def changeAccountCredentialsImmediately(self, accountId):

        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None

        headers = {
            'Content-Type': 'application/json',
            'Authorization': self.token
        }
        odata = {
            'ChangeEntireGroup': True
        }
        sdata = json.dumps(odata)

        try:
            response = requests.post('{0}/api/Accounts/{1}/Change'.format(self.address, accountId), data=sdata, headers=headers, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                #print(response.status_code, flush = True)
                #print(response.text, flush = True)
                return {}
        except Exception as e:
            self.lastexception = e
            return None

    def deleteAccount(self, accountId):

        if self.dorelogin:
            now = datetime.now()
            total_seconds = (now - self.lastlogon).total_seconds()
            if (total_seconds > self.doreloginwindow):
                self.logoff()
                b = self.login_cyberark()
                if not b:
                    return None

        headers = {
            'Authorization': self.token
        }

        try:
            response = requests.delete('{0}/api/Accounts/{1}'.format(self.address, accountId), headers=headers, verify=self.verify, timeout=self.timeout)
            if self.sleepafter > 0:
                time.sleep(self.sleepafter)
            if response.status_code >= 400:
                raise Exception(response.text)
            else:
                return {}
        except Exception as e:
            self.lastexception = e
            return None
