import sys
import os
import shutil
import socket
import json
import time
import re
import copy
from ireportutil import IReportUtil
from config import config
from datetime import datetime
from cyberark.vault import Vault

mode = config['settings']['mode']
vault = None

sprm_aim_appid = 'AppDNA'
sprm_aim_safe = 'S-PAM'
sprm_aim_object = 'Operating%20System-UnixSSH-BoKS-erpmusr'

o_win_cr_aimcredential = None

ds_platform = {
    'unix': [
        'cmdb_ci_aix_server',
        'cmdb_ci_linux_server',
        'cmdb_ci_hpux_server',
        'cmdb_ci_solaris_server'
    ],
    'windows': [
        'cmdb_ci_win_server'
    ],
    'other': [
        'cmdb_ci_esx_server',
        'cmdb_ci_hyper_v_server',
        'cmdb_ci_ibm_frame',
        'cmdb_ci_ibm_zos_server',
        'cmdb_ci_netware_server',
        'cmdb_ci_hmc_server',
    ]
}

ds_files = []

ds_segmento = {
    'Varejo': {
        'output': {
            'unix': 'DNALinVarejo.csv',
            'windows': 'DNAWinVarejo.csv'
        },
        'domains': [
            'corp1.rc.itau',
            'corp.ihf',
            'itau.corp.ihf',
            'itauint.corp.ihf',
            'itauage.corp.ihf',
            'servicos.corp.ihf',
            'corp15.rc.itau',
            'srae1.rae.itau',
            'previtec.ihf',
            'corretora.ihf',
            'corp6.rc.itau',
            'srsc2.rsc.itau'
        ]
    },
    'Atacado': {
        'output': {
            'unix': 'DNALinAtacado.csv',
            'windows': 'DNAWinAtacado.csv'
        },
        'domains': [
            'NT.corp.bba.com.br',
            'corp.bba.com.br',
            'mundostar.com.uy'
        ]
    },
    'Cartões': {
        'output': {
            'unix': 'DNALinCartoes.csv',
            'windows': 'DNAWinCartoes.csv'
        },
        'domains': [
            'orbitall.corp',
            'corp'
        ]
    },
    'Cartões': {
        'output': {
            'unix': 'DNALinCartoes.csv',
            'windows': 'DNAWinCartoes.csv'
        },
        'domains': [
            'orbitall.corp',
            'corp'
        ]
    },
    'Cartoes': {
        'output': {
            'unix': 'DNALinCartoes.csv',
            'windows': 'DNAWinCartoes.csv'
        },
        'domains': [
            'orbitall.corp',
            'corp'
        ]
    },
    'Rede': {
        'output': {
            'unix': 'DNALinRede.csv',
            'windows': 'DNAWinRede.csv'
        },
        'domains': [
            'redecardsa.rccorp.net',
            'redecard.corp',
            'redepf.ihf',
            'rccorp.net'
        ]
    },
    'Cloud': {
        'output': {
            'unix': 'DNALinCloudPrivada.csv',
            'windows': 'DNAWinCloudPrivada.csv'
        },
        'domains': [
            'ctmm2.prod.cloud.ihf',
            'ctmm1.prod.cloud.ihf',
            'ctsp.prod.cloud.ihf'
        ]
    },
    'Analytics': {
        'output': {
            'unix': 'DNALinDwbi.csv',
            'windows': ''
        },
        'domains': [
        ]
    }
}

ds_map = {
    'unix': {
        'Machine_Name': '',
        'Account_Name': '',
        'Local_Mapped_Account': '',
        'Account_Display_Name': '',
        'Account_Type': '',
        'Account_Category': '',
        'Account_Group': '',
        'Account_Description': '',
        'Compliance_Status': '',
        'Account_State': '',
        'Password_Never_Expires': '',
        'Password_Age': '',
        'Password_Last_Set': '',
        'Insecure_Privilege_Escalation': '',
        'Insecure_Privilege_Escalation_Reason': '',
        'Last_Login_Date': '',
        'Account_Expiration_Date': '',
        'OS_Version': '',
        'Details': '',
        'dominio': ''
    },
    'windows': {
        'Machine_Name': '',
        'Machine_Type': '',
        'Account_Name': '',
        'Account_Display_Name': '',
        'Account_Type': '',
        'Account_Category': '',
        'Account_Group': '',
        'Privileged_Domain_Group': '',
        'Pass_the_Hash_Vulnerable': '',
        'Pass_the_Hash_Hash_Found': '',
        'Causes_Vulnerability_On_of_Machines': '',
        'Threat_Cause': '',
        'Service_Account_Type': '',
        'Service_Account_Description': '',
        'Compliance_Status': '',
        'Account_State': '',
        'Password_Never_Expires': '',
        'Password_Age': '',
        'Password_Last_Set': '',
        'Last_Login_Date': '',
        'Account_Expiration_Date': '',
        'OS_Version': '',
        'Details': '',
        'dominio': ''
    }
}

stats = {
    'scan': {
        'totals': {
            'total': 0,
            'success': 0,
            'failure': 0,
            'platform': {
                'unix': {
                    'total': 0,
                    'success': 0,
                    'failure': 0
                },
                'windows': {
                    'total': 0,
                    'success': 0,
                    'failure': 0
                }
            }
        },
        'success': {
            'platform': {
                'unix': {
                    'machines': [],
                    'accounts': {
                        'normal': 0,
                        'privileged': 0
                    }
                },
                'windows': {
                    'machines': [],
                    'accounts': {
                        'normal': 0,
                        'privileged': 0
                    }
                },
            },
            'accounts': {
                'normal': 0,
                'privileged': 0
            }
        },
        'failure': {
            'platform': {
                'unix': {
                    'machines': []
                },
                'windows': {
                    'machines': []
                }
            }
        }
    }
}

util = IReportUtil()

def getPlatformByClasse(classe):
    for platform in ds_platform:
        if classe in ds_platform[platform]:
            return platform

def main():

    renewtokentime = 120

    inputfilepath = 'files/in.csv'
    executionname = 'output'
    segmentos = ['Varejo', 'Atacado', 'Cloud', 'Rede', 'Analytics', 'Cartões', 'Cartoes']
    plataformas = ['unix', 'windows']

    if sys.argv.__len__() > 1:
        inputfilepath = sys.argv[1]

    if sys.argv.__len__() > 2:
        executionname = sys.argv[2]

    if sys.argv.__len__() > 3:
        ssegmentos = sys.argv[3]
        segmentos = ssegmentos.split(',')

    if sys.argv.__len__() > 4:
        splataformas = sys.argv[4]
        plataformas = splataformas.split(',')

    if os.path.isfile(executionname):
        os.remove(executionname)

    if os.path.isdir(executionname):
        shutil.rmtree(executionname)

    time.sleep(1)

    os.makedirs(executionname)

    if mode == 'vault':
        vault = Vault(config['vault']['address'])
        vault.setVerify(config['vault']['verify'])
        vault.setTimeout(config['vault']['requesttimeout'])
        vault.setSleepAfter(config['vault']['sleepafter'])
        vault.setDoRelogin(True)
        vault.setDoReloginWindow(60)
        cred_username = ''
        cred_password = ''
        if 'aim' in config['vault'] and 'enabled' in config['vault']['aim'] and config['vault']['aim']['enabled']:
            aimcredential = vault.getAIMCredential(config['vault']['aim']['address'], config['vault']['aim']['appid'], config['vault']['aim']['safe'], config['vault']['aim']['object'])
            if aimcredential is None:
                print("Failed to retrieve credential from AIM", flush = True)
                print(vault.getLastException(), flush = True)
                print("Failed to retrieve credential from AIM", file=outputFileTrace, flush = True)
                print(vault.getLastException(), file=outputFileTrace, flush = True)
                exit(1)
            cred_username = aimcredential['UserName']
            cred_password = aimcredential['Content']
        else:
            cred_username = config['vault']['auth']['cyberark']['username']
            cred_password = config['vault']['auth']['cyberark']['password']
        b = vault.login_cyberark(cred_username, cred_password)
        if not b:
            # Falha de Login
            #writeStatusLog(outputFileNotSuccess, 'ERROR', '', '', '', 'ERROR: Failed to connect to Cyberark')
            print("ERROR: Failed to connect to Cyberark", file=outputFileTrace, flush = True)
            print(vault.getLastException(), file=outputFileTrace, flush = True)
            print("ERROR: Failed to connect to Cyberark", flush = True)
            print(vault.getLastException(), flush = True)
            exit(1)
        print("LOGADO NO CYBERARK", flush = True)
    linecount = 0
    #with open(inputfilepath, 'r', encoding="cp1252") as inputFileSummary:
    with open(inputfilepath, 'r', encoding="utf-8") as inputFileSummary:
        line = inputFileSummary.readline()
        while True:
            line = inputFileSummary.readline()
            if not line:
                break
            linecount = linecount + 1

    #with open(inputfilepath, 'r', encoding="cp1252") as inputFile:
    with open(inputfilepath, 'r', encoding="utf-8") as inputFile:
        line = inputFile.readline()
        i = 1
        while True:
            line = inputFile.readline()
            if not line:
                break
            aline = line.split(';')
            campo_nome = aline[0]
            campo_ip = aline[1]
            campo_ambiente = aline[2]
            campo_sistema_operacional = aline[3]
            campo_segmento_de_negocio = aline[4]
            campo_funcao = aline[5]
            campo_classe = aline[6]
            campo_os_domain = aline[12]
            campo_dns_domain = aline[13]
            platform = getPlatformByClasse(campo_classe)
            if platform is None:
                platform = 'other'
            if platform not in plataformas:
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}) : PLATAFORMA NAO LISTADA : [{4}] : [{5}]'.format(campo_nome, campo_ip, i, linecount, platform, plataformas), flush = True)
                i = i + 1
                continue
            if platform == 'other':
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}) : PLATAFORMA OTHER'.format(campo_nome, campo_ip, i, linecount), flush = True)
                i = i + 1
                continue
            if campo_segmento_de_negocio not in segmentos:
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}) : SEGMENTO DE NEGOCIO NAO LISTADO'.format(campo_nome, campo_ip, i, linecount), flush = True)
                i = i + 1
                continue
            if campo_segmento_de_negocio not in ds_segmento:
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}) : SEGMENTO DE NEGOCIO NAO MAPEADO'.format(campo_nome, campo_ip, i, linecount), flush = True)
                i = i + 1
                continue
            if campo_ambiente not in ['Production','DR','Pre-Production','PRODUCAO']:
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}) : AMBIENTE NAO CONSIDERADO'.format(campo_nome, campo_ip, i, linecount), flush = True)
                i = i + 1
                continue
            if "DOMAIN CONTROLLER" in campo_funcao.upper():
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}) : DOMAIN CONTROLLER'.format(campo_nome, campo_ip, i, linecount), flush = True)
                i = i + 1
                continue

            filepath = '{0}/{1}'.format(executionname, ds_segmento[campo_segmento_de_negocio]['output'][platform])
            print('PROCESSING MACHINE {0} [{1}] ({2} of {3})'.format(campo_nome, campo_ip, i, linecount), flush = True)
            sresult = processMachine(executionname, platform, filepath, campo_nome, campo_ip, campo_dns_domain, vault, aline)
            if sresult != '':
                print('SKIPPING MACHINE {0} [{1}] ({2} of {3}): {4}'.format(campo_nome, campo_ip, i, linecount, sresult), flush = True)
            i = i + 1

def saveStats(executionname):
    #print('{0}/stats.json'.format(executionname), flush = True)
    with open('{0}/stats.json'.format(executionname), mode='w', encoding='utf-8') as statFile:
        print(json.dumps(stats, indent=4), file=statFile, flush=True)

def getLogonCredential_env(platform, machine, aline):
    oenv = None
    with open('env.json', mode='r', encoding='utf-8') as envfile:
        sdata = envfile.read()
        oenv = json.loads(sdata)
    if platform == 'unix':
        return oenv['unix']
    elif platform == 'windows':
        return oenv['windows']
    else:
        return {
            'username': '',
            'password': ''
        }

def getMachine_env(platform, domain, machine, ovault):
    oenv = None
    with open('env.json', mode='r', encoding='utf-8') as envfile:
        sdata = envfile.read()
        oenv = json.loads(sdata)
    if oenv is not None and platform in oenv and 'machine' in oenv[platform] and oenv[platform]['machine'] is not None:
        return oenv[platform]['machine']
    else:
        return machine

def buildSafeNameForUnix(aline):
    campo_nome = aline[0]
    campo_ip = aline[1]
    campo_ambiente = aline[2]
    campo_sistema_operacional = aline[3]
    campo_segmento_de_negocio = aline[4]
    campo_funcao = aline[5]
    campo_classe = aline[6]
    campo_os_domain = aline[12]
    campo_dns_domain = aline[13]
    campo_grupo_suporte = aline[17]
    actual_ambiente = 'P'
    if campo_ambiente in ['Development', 'Experimentation']:
        actual_ambiente = 'D'
    elif campo_ambiente in ['Homologation']:
        actual_ambiente = 'H'
    elif campo_ambiente in ['Production', 'DR', 'Pre-Production', 'Contingency']:
        actual_ambiente = 'P'
    safeName = 'S-DNAEO8C'
    if campo_segmento_de_negocio == 'Cloud':
        safeName = '{0}-CLD-LNX-LOGON'.format(actual_ambiente)
    elif campo_grupo_suporte.lower() == 'Suporte Ferramentas de Big Data'.lower() and campo_classe == 'cmdb_ci_linux_server':
        safeName = '{0}-BDT-LNX-LOGON'.format(actual_ambiente)
    elif campo_classe in ['cmdb_ci_linux_server', 'cmdb_ci_unix_server']:
        safeName = '{0}-CONV-LNX-LOGON'.format(actual_ambiente)
    elif campo_classe in ['cmdb_ci_aix_server', 'cmdb_ci_solaris_server']:
        safeName = '{0}-AIX-SOLARIS-LOGON'.format(actual_ambiente)
    #print(safeName, flush = True)     
    return safeName

def getLogonCredential_cyberark(platform, domain, machine, ovault, aline):
    result = None
    cacc = None
    is_aim = False
    aim_credential = None
    #print('teste3:', machine, flush = True)
    if platform == 'unix':
        acc = ovault.searchAccountsOnSafe('LOGON {0}'.format(machine), buildSafeNameForUnix(aline), limit = 100)
        if acc is not None and 'value' in acc and acc['value'] is not None and acc['value'].__len__() > 0:
            cacc = acc['value'][0]
        is_aim = False
    elif aline[4] in ['Cartões', 'Cartões', 'Cartoes', 'Rede']:
        if o_win_cr_aimcredential is None:
            o_win_cr_aimcredential = vault.getAIMCredential(config['vault']['aim']['address'], sprm_aim_appid, sprm_aim_safe, sprm_aim_object)
        aimcredential = o_win_cr_aimcredential
        is_aim = True
    else:
        acc = ovault.searchAccountsOnSafe('DNAEO8C {0}'.format(domain), 'S-DNAEO8C', limit = 100)
        if acc is not None and 'value' in acc and acc['value'] is not None and acc['value'].__len__() > 0:
            cacc = acc['value'][0]
        is_aim = False
    if cacc is not None:
        if is_aim:
            if aimcredential is None:
                print("Failed to retrieve credential from AIM for Windows Login on REDE and CARTOES", flush = True)
                print(vault.getLastException(), flush = True)
                print("Failed to retrieve credential from AIM for Windows Login on REDE and CARTOES", file=outputFileTrace, flush=True)
                print(vault.getLastException(), file=outputFileTrace, flush=True)
                exit(1)
            cred_username = aimcredential['UserName']
            cred_password = aimcredential['Content']
            result = {
                'username': cred_username,
                'password': cred_password
            }
        else:
            accountId = cacc['id']
            password = ovault.retrievePassword(accountId)
            result = {
                'username': cacc['userName'],
                'password': password
            }
    return result

domainmap = {}

def getLogonCredential(platform, domain, machine, ovault, aline):
    if platform == 'windows':
        if domain in domainmap :
            return domainmap[domain]
    credential = None
    if mode == 'env':
        credential = getLogonCredential_env(platform, domain, machine, ovault, aline)
    elif mode == 'vault':
        credential = getLogonCredential_cyberark(platform, domain, machine, ovault, aline)
    if platform == 'windows':
        domainmap[domain] = credential
    return credential

def processMachine(executionname, platform, filepath, machine, ip, domain, ovault, aline):
    ostatmachine = {
        'name': machine,
        'message': '',
        'accounts': {
            'normal': {
                'count': 0,
                'names': [],
                'users': []
            },
            'privileged': {
                'count': 0,
                'names': [],
                'users': []
            }
        }
    }
    credential = getLogonCredential(platform, domain, machine, ovault, aline)
    if credential is None:
        stats['scan']['totals']['failure'] = stats['scan']['totals']['failure'] + 1
        stats['scan']['totals']['platform']['unix']['failure'] = stats['scan']['totals']['platform']['unix']['failure'] + 1
        ostatmachine['message'] = 'CREDENTIAL NOT FOUND'
        status = 'failure'
        stats['scan'][status]['platform'][platform]['machines'].append(ostatmachine)
        return 'CREDENTIAL NOT FOUND'
    smachine = machine
    if mode == 'env':
        smachine = getMachine_env(platform, domain, machine, ovault)
    else:
	    smachine = ip
    stats['scan']['totals']['total'] = stats['scan']['totals']['total'] + 1

    if platform == 'unix':
        port = 4098
        if aline[4] == 'Cloud':
            port = 22		
        stats['scan']['totals']['platform']['unix']['total'] = stats['scan']['totals']['platform']['unix']['total'] + 1
        users = util.accountreport_unix(smachine, port, credential['username'], credential['password'])
        status = ''
        if isinstance(users, str):
            stats['scan']['totals']['failure'] = stats['scan']['totals']['failure'] + 1
            stats['scan']['totals']['platform']['unix']['failure'] = stats['scan']['totals']['platform']['unix']['failure'] + 1
            ostatmachine['message'] = users
            status = 'failure'
        elif users is None:
            stats['scan']['totals']['failure'] = stats['scan']['totals']['failure'] + 1
            stats['scan']['totals']['platform']['unix']['failure'] = stats['scan']['totals']['platform']['unix']['failure'] + 1
            ostatmachine['message'] = 'Unexpected Error'
            status = 'failure'
        else:
            stats['scan']['totals']['success'] = stats['scan']['totals']['success'] + 1
            stats['scan']['totals']['platform']['unix']['success'] = stats['scan']['totals']['platform']['unix']['success'] + 1
            status = 'success'
        records = []
        if status == 'success':
            for user in users:
                category = ''
                if user['type'] == 'normal':
                    category = 'Non-Privileged Shared'
                elif user['type'] == 'privileged':
                    category = 'Privileged Local'
                stats['scan']['success']['platform']['unix']['accounts'][user['type']] = stats['scan']['success']['platform']['unix']['accounts'][user['type']] + 1
                stats['scan']['success']['accounts'][user['type']] = stats['scan']['success']['accounts'][user['type']] + 1
                ostatmachine['accounts'][user['type']]['count'] = ostatmachine['accounts'][user['type']]['count'] + 1
                ostatmachine['accounts'][user['type']]['names'].append(user['username'])
                ostatmachine['accounts'][user['type']]['users'].append(user)
                for group in user['groups']:
                    orecord = copy.copy(ds_map['unix'])
                    orecord['Machine_Name'] = machine
                    orecord['Account_Name'] = user['username']
                    orecord['Account_Category'] = category
                    orecord['Account_Group'] = group
                    orecord['Password_Last_Set'] = user['passwordlastset']
                    orecord['Account_Expiration_Date'] = user['expires']
                    orecord['dominio'] = domain
                    records.append(orecord)
        stats['scan'][status]['platform']['unix']['machines'].append(ostatmachine)
        with open(filepath, mode='a', encoding='utf-8') as outputFile:
            if filepath not in ds_files:
                ds_files.append(filepath)
                aline = []
                for key in ds_map['unix']:
                    aline.append(key)
                sline = '|'.join(aline)
                print(sline, file=outputFile, flush=True)
            for record in records:
                aline = []
                for key in record:
                    aline.append(record[key])
                sline = '|'.join(aline)
                print(sline, file=outputFile, flush=True)
    elif platform == 'windows':
        stats['scan']['totals']['platform']['windows']['total'] = stats['scan']['totals']['platform']['windows']['total'] + 1
        users = util.accountreport_windows(smachine, credential['username'] + '@' + domain, credential['password'])
        if isinstance(users, str):
            stats['scan']['totals']['failure'] = stats['scan']['totals']['failure'] + 1
            stats['scan']['totals']['platform']['windows']['failure'] = stats['scan']['totals']['platform']['windows']['failure'] + 1
            ostatmachine['message'] = users
            status = 'failure'
        elif users is None:
            stats['scan']['totals']['failure'] = stats['scan']['totals']['failure'] + 1
            stats['scan']['totals']['platform']['windows']['failure'] = stats['scan']['totals']['platform']['windows']['failure'] + 1
            ostatmachine['message'] = 'Unexpected error'
            status = 'failure'
        else:
            stats['scan']['totals']['success'] = stats['scan']['totals']['success'] + 1
            stats['scan']['totals']['platform']['windows']['success'] = stats['scan']['totals']['platform']['windows']['success'] + 1
            status = 'success'
        records = []
        if status == 'success':
            for user in users:
                category = ''
                if user['type'] == 'normal':
                    category = 'Non-Privileged Shared'
                elif user['type'] == 'privileged':
                    category = 'Privileged Local'
                stats['scan']['success']['platform']['windows']['accounts'][user['type']] = stats['scan']['success']['platform']['windows']['accounts'][user['type']] + 1
                stats['scan']['success']['accounts'][user['type']] = stats['scan']['success']['accounts'][user['type']] + 1
                ostatmachine['accounts'][user['type']]['count'] = ostatmachine['accounts'][user['type']]['count'] + 1
                ostatmachine['accounts'][user['type']]['names'].append(user['username'])
                ostatmachine['accounts'][user['type']]['users'].append(user)
                for group in user['groups']:
                    orecord = copy.copy(ds_map['windows'])
                    orecord['Machine_Name'] = machine
                    orecord['Account_Name'] = user['username']
                    orecord['Account_Category'] = category
                    orecord['Account_Group'] = group
                    orecord['Password_Last_Set'] = user['passwordlastset']
                    orecord['Account_Expiration_Date'] = user['expires']
                    orecord['dominio'] = domain
                    records.append(orecord)
        stats['scan'][status]['platform']['windows']['machines'].append(ostatmachine)
        with open(filepath, mode='a', encoding='utf-8') as outputFile:
            if filepath not in ds_files:
                ds_files.append(filepath)
                aline = []
                for key in ds_map['windows']:
                    aline.append(key)
                sline = '|'.join(aline)
                print(sline, file=outputFile, flush=True)
            for record in records:
                aline = []
                for key in record:
                    aline.append(record[key])
                sline = '|'.join(aline)
                print(sline, file=outputFile, flush=True)
    saveStats(executionname)
    return ''

# Funcao que realiza um teste de porta, e retorna um booleano True sucesso, False falha
def isOpenPort(address, port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((address, int(port)))
        s.shutdown(2)
        return True
    except Exception as ex:
        return False

def getFromBasePath(path):
    sep = '/'
    if config['settings']['basedir'].endswith('/'):
        sep = ''
    return '{0}{1}{2}'.format(config['settings']['basedir'], sep, path)

def test():
    print(flush = True)
    print('MACHINE = PEACESTAR (UNIX)', flush = True)
    users = util.accountreport_unix("peacestar", 22, "root", "********")
    print(users, flush = True)
    print(flush = True)
    print('MACHINE = SNEPRPM04V (WINDOWS)', flush = True)
    users = util.accountreport_windows("SNEPRPM04V", "COMET\\svc_admin", "********")
    print(users, flush = True)

main()
#test()
