import re;
import socket;
from ipaddress import ip_network


class DiscoveryExplorer:

    def __init__(self, defaulttimeout=None):
        if defaulttimeout is None:
            self._defaulttimeout = 0.01;
        else:
            self._defaulttimeout = defaulttimeout;
        pass;

    def testPort(self, address, port, timeout=None):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if not timeout is None:
            sock.settimeout(timeout);
        else:
            sock.settimeout(self._defaulttimeout);
        result = sock.connect_ex((address, port))
        if result == 0:
            return True;
        else:
            return False;

    def testPorts(self, address, ports):
        result = True;
        for port in ports:
            result = result and self.testPort(address, port);
        return result;

    def guessOS(self, address='localhost'):
        if self.testPorts(address, [22]):
            return 'linux';
        elif self.testPorts(address, [135, 139, 445]):
            return 'windows';
        else:
            return None;

    def guessApps(self, address=''):
        apps = [];
        so = self.guessOS(address);
        if so is not None:
            apps.append(["os", so]);
        if self.testPorts(address, [1433]):
            apps.append(["app", "mssql"]);
        if self.testPorts(address, [3306]):
            apps.append(["app", "mysql"]);
        return apps;

    def expandNetworkMask(self, masks):
        ips = [];
        masks = unicode(masks); 
        amasks = masks.split(',');
        for mask in amasks:
            if re.match('^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$', mask) is not None or re.match('^([0-9a-fA-F]{4}|0)(\:([0-9a-fA-F]{4}|0)){7}$', mask) is not None:
                ips.append(mask);
            elif re.match('\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(/\d{1,2})', mask) is not None:
                aoips = list(ip_network(mask).hosts());
                print aoips;
                for oip in aoips:
                    ips.append(str(oip.exploded));
            else:
                ssystemname = mask;
                if '.' in ssystemname:
                    ssystemnamefull = ssystemname;
                    ssystemipaddress = socket.gethostbyname(ssystemnamefull);
                    ssystemnameshort = ssystemnamefull.split('.')[0];
                else:
                    ssystemnameshort = ssystemname;
                    ssystemipaddress = socket.gethostbyname(ssystemnameshort);
                    ssystemnamefull = socket.getfqdn(ssystemipaddress);
                ips.append(ssystemipaddress);
        return ips;

    def getIPmoreInfo(self, st):
        ssystemipaddress = st;
        ssystemnameshort = st;
        ssystemnamefull = st;
        try:
            if re.match('\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', st) is not None or re.match('^([0-9a-fA-F]{4}|0)(\:([0-9a-fA-F]{4}|0)){7}$', st) is not None:
                ssystemipaddress = st;
                host = socket.gethostbyaddr(ssystemipaddress);
                if '.' in host[0]:
                    ssystemnamefull = host[0];
                    ssystemnameshort = ssystemnamefull.split('.')[0];
                else:
                    ssystemnamefull = socket.getfqdn(host[0]);
                    ssystemnameshort = ssystemnamefull.split('.')[0];
            else:
                ssystemname = st;
                if '.' in ssystemname:
                    ssystemnamefull = ssystemname;
                    ssystemipaddress = socket.gethostbyname(ssystemnamefull);
                    ssystemnameshort = ssystemnamefull.split('.')[0];
                else:
                    ssystemnameshort = ssystemname;
                    ssystemipaddress = socket.gethostbyname(ssystemnameshort);
                    ssystemnamefull = socket.getfqdn(ssystemipaddress);
        except:
            ssystemipaddress = st;
            ssystemnameshort = st;
            ssystemnamefull = st;
        result = {
            'ip': ssystemipaddress,
            'shortname': ssystemnameshort,
            'fullname': ssystemnamefull
        }
        return result;

    def discoverFromMask(self, mask):
        ips = self.expandNetworkMask(mask);
        oassets = [];
        for ip in ips:
            # os = self.guessOS(ip);
            apps = self.guessApps(ip);
            if apps is None or apps.__len__() <= 0:
                continue;
            oIPmoreInfo = self.getIPmoreInfo(ip);
            # if not os is None:
            for app in apps:
                if app[0] == 'os':
                    os = app[1];
                    print "Trying discovery of [{0}][OS: {1}] ...".format(ip, os);
                    try:
                        oasset = {
                            'ip': oIPmoreInfo['ip'],
                            'shortname': oIPmoreInfo['shortname'],
                            'fullname': oIPmoreInfo['fullname'],
                            'type': 'os',
                            'os': os 
                        };
                        oassets.append(oasset);
                        print "Successful discovery of [{0}][OS: {1}]!".format(ip, os);
                    except Exception as ex:
                        print "An exception of type {0} occurred. Arguments:\n{1}".format(type(ex).__name__, ex.args);
                elif app[0] == 'app':
                    if app[1] == 'mssql' or app[1] == 'mysql':
                        print "Trying discovery of [{0}][OS: {1}] ...".format(ip, app[1]);
                        try:
                            oasset = {
                                'ip': oIPmoreInfo['ip'],
                                'shortname': oIPmoreInfo['shortname'],
                                'fullname': oIPmoreInfo['fullname'],
                                'type': 'app',
                                'app': app[1]
                            };
                            oassets.append(oasset);
                            print "Successful discovery of [{0}][APP: {1}]!".format(ip, app[1]);
                        except Exception as ex:
                            print "An exception of type {0} occurred. Arguments:\n{1}".format(type(ex).__name__, ex.args);
        return oassets;


def getDomainDistinguishedName(domainname):
    adomainname = domainname.split('.');  # list(reversed(domainname.split('.')));
    aprefixeddomainname = ['dc=' + s for s in adomainname];
    sdistinguishedname = ','.join(aprefixeddomainname);
    return sdistinguishedname;

# import nmap;
# opscanner = nmap.PortScanner();
# opscanner.scan('coldstar.comet.egypteam.com');
# print opscanner.csv('coldstar.comet.egypteam.com');
