import json;
import requests;
from collections import namedtuple;

def _json_object_hook(d): return namedtuple('Object', d.keys())(*d.values());

def json2obj(data): return json.loads(data, object_hook=_json_object_hook);

class Object:
    pass;

class Onepoint:
    def __init__(self, address = None, email = None, password = None, cert = None, verify = False, jsondecodemode = 'dict'):
        self._jsondecodemode = jsondecodemode;
        self._address = address;
        self._email = email;
        self._password = password;
        self._token = None;
        self._cert = cert;
        self._verify = verify;
    @staticmethod
    def fromGbl(gbl, cert = None, verify = False):
        op = Onepoint();
        op._address = gbl.address;
        op._email = None;
        op._password = None;
        op._token = gbl.session.id;
        op._cert = cert;
        op._verify = verify;
        return op;
    def login(self):
        headers = {
            'x-pantufa-auth-email': self._email,
            'x-pantufa-auth-password': self._password
        }
        queryparams = {
        }
        response = requests.get(self._address + 'api/authentication/login', headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
            raise ValueError('Authentication error');
        oresponse = json2obj(response.text);
        if oresponse is None or oresponse.status >= 400 or oresponse.data is None or oresponse.data.session is None:
            raise ValueError('Authentication error');
        self._token = oresponse.data.session.id;
    def _assert_authenticated(self):
        if self._token is None:
            raise ValueError('Request Error: Not authenticated');
    def getCredentials(self):
        self._assert_authenticated();
        headers = {
            'x-pantufa-auth-token': self._token
        }
        queryparams = {
        }
        response = requests.get(self._address + 'api/vault', headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
                raise ValueError('HTTP Request error');
        if self._jsondecodemode == 'namedtuple':
            oresponse = json2obj(response.text);
            if oresponse is None or oresponse.status >= 400 or oresponse.data is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse.data;
        else:
            oresponse = json.dumps(response.text);
            if oresponse is None or oresponse['status'] >= 400 or oresponse['data'] is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse['data'];
    def getCredential(self, path, backend = None):
        self._assert_authenticated();
        headers = {
            'x-pantufa-auth-token': self._token
        }
        queryparams = {}
        if not backend is None:
            queryparams['backend'] = backend;
        #print self._address + 'api/vault' + path + '?backend=' + backend + '&full=true';
        response = requests.get(self._address + 'api/vault' + path, headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
            raise ValueError('HTTP Request error');
        if self._jsondecodemode == 'namedtuple':
            oresponse = json2obj(response.text);
            if oresponse is None or oresponse.status >= 400 or oresponse.data is None:
                raise ValueError('Request error: ' + response.text);
            if oresponse.data.__len__() == 0:
                #raise ValueError('Credential not found!');
                return None;
            #if oresponse.data.__len__() > 1:
            #    raise ValueError('More than one credential matched!');
            return oresponse.data[0];
        else:
            oresponse = json.loads(response.text);
            if oresponse is None or oresponse['status'] >= 400 or oresponse['data'] is None:
                raise ValueError('Request error: ' + response.text);
            if oresponse['data'].__len__() == 0:
                #raise ValueError('Credential not found!');
                return None;
            #if oresponse.data.__len__() > 1:
            #    raise ValueError('More than one credential matched!');
            return oresponse['data'][0];
    def postCredential(self, path, ocredential, backend = None):
        self._assert_authenticated();
        headers = {
            'x-pantufa-auth-token': self._token
        }
        queryparams = {}
        if not backend is None:
            queryparams['backend'] = backend;
        data = json.dumps(ocredential);
        response = requests.post(self._address + 'api/vault' + path, data = data, headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
            raise ValueError('HTTP Request error');
        if self._jsondecodemode == 'namedtuple':
            oresponse = json2obj(response.text);
            if oresponse is None or oresponse.status >= 400 or oresponse.data is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse.data;
        else:
            oresponse = json.loads(response.text);
            if oresponse is None or oresponse['status'] >= 400 or oresponse['data'] is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse['data'];
    def getAsset(self, path, backend = None):
        self._assert_authenticated();
        headers = {
            'x-pantufa-auth-token': self._token
        }
        queryparams = {}
        if not backend is None:
            queryparams['backend'] = backend;
        #print self._address + 'api/vault' + path + '?backend=' + backend + '&full=true';
        response = requests.get(self._address + 'api/asset' + path, headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
            raise ValueError('HTTP Request error');
        if self._jsondecodemode == 'namedtuple':
            oresponse = json2obj(response.text);
            if oresponse is None or oresponse.status >= 400 or oresponse.data is None:
                raise ValueError('Request error: ' + response.text);
            if oresponse.data.__len__() == 0:
                #raise ValueError('Credential not found!');
                return None;
            #if oresponse.data.__len__() > 1:
            #    raise ValueError('More than one credential matched!');
            return oresponse.data[0];
        else:
            oresponse = json.loads(response.text);
            if oresponse is None or oresponse['status'] >= 400 or oresponse['data'] is None:
                raise ValueError('Request error: ' + response.text);
            if oresponse['data'].__len__() == 0:
                #raise ValueError('Credential not found!');
                return None;
            #if oresponse.data.__len__() > 1:
            #    raise ValueError('More than one credential matched!');
            return oresponse['data'][0];
    def getAssets(self, path, backend = None):
        self._assert_authenticated();
        headers = {
            'x-pantufa-auth-token': self._token
        }
        queryparams = {}
        if not backend is None:
            queryparams['backend'] = backend;
        #print self._address + 'api/vault' + path + '?backend=' + backend + '&full=true';
        response = requests.get(self._address + 'api/asset' + path, headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
            raise ValueError('HTTP Request error');
        if self._jsondecodemode == 'namedtuple':
            oresponse = json2obj(response.text);
            if oresponse is None or oresponse.status >= 400 or oresponse.data is None:
                raise ValueError('Request error: ' + response.text);
            if oresponse.data.__len__() == 0:
                #raise ValueError('Credential not found!');
                return None;
            #if oresponse.data.__len__() > 1:
            #    raise ValueError('More than one credential matched!');
            return oresponse.data[0];
        else:
            oresponse = json.loads(response.text);
            if oresponse is None or oresponse['status'] >= 400 or oresponse['data'] is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse['data'];
    def postAsset(self, path, oasset, backend = None):
        self._assert_authenticated();
        headers = {
            'x-pantufa-auth-token': self._token
        }
        queryparams = {}
        if not backend is None:
            queryparams['backend'] = backend;
        data = json.dumps(oasset);
        response = requests.post(self._address + 'api/asset' + path, data = data, headers = headers, params = queryparams, verify=self._verify, cert=self._cert);
        if response is None or response.status_code >= 400:
            raise ValueError('HTTP Request error');
        if self._jsondecodemode == 'namedtuple':
            oresponse = json2obj(response.text);
            if oresponse is None or oresponse.status >= 400 or oresponse.data is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse.data;
        else:
            oresponse = json.loads(response.text);
            if oresponse is None or oresponse['status'] >= 400 or oresponse['data'] is None:
                raise ValueError('Request error: ' + response.text);
            return oresponse['data'];
    def ____TEST(self):
        #response = requests.post(self.address + '/Login', data = spayload, headers = myheaders, params = queryparams, verify=self._verify, cert=self._cert);
        pass;
